{  METER VGA display.spin

          **************************************
                     METER display
          **************************************
                    Peter Jakacki 2011
          **************************************

Hardware: CE0972 Widget (or demoboard)
Uses:     VGA port via RJ45 adaptor
          PS/2 port clock for capacitance probe

Desciption:
Measure capacitance using a single I/O pin with a 10K pullup
Filter, range and display results on VGA screen with large characters

For the demo board just change the clock and PLL and choose an I/O pin
Don't forget to add a 10K pullup and optionally a 100R in series with the I/O line
to minimize discharge current.The discharge time is preset to 10ms but could be made
adjustable if necessary.

                                VCC
                                 |
                                 10K
I/O PIN----------------100R------|-------> CAPACITOR <-----GND

}

'#define demoboard              ' <-------------- uncomment this for the demoboard

DAT

          long $110811_00        ' autoload version check YYMMDDvv

VER
'         byte cr,lf, " HARDWARE: CE0972 WIDGET"
          byte cr,lf," FIRMWARE: CAPMETER VGA DISPLAY"
          byte cr,lf," V1.0 110811-0000  "
          byte cr,lf,0



pub start | ch,x,y,i,h,sample,g,avg
   screen.start(video)
   cognew(@Measure,@interval)
   screen.pen(1)
   repeat i from 0 to screen#tiles - 1                  'init tile colors to white on dark blue
     screen.color(i,$ff04)

' meter colors  WHITE on RED 128 lines high from the bottom
   repeat i from 128 to 192
     screen.color(i,$ffC0)
   screen.pen(1)
   screen.scale(1,2)
   screen.text(90,8,string("PBJ Capacitance Meter"))
   screen.box(78,4,434,80)
   screen.scale(1,1)
   screen.text(376,140,string("Cap code"))

   '*************************** main loop ********************
   repeat
     sample := interval
                                    ' diagnostic indicator in top right hand corner
     if status and interval < 25_000_000 and interval > 20
       sample := ||(avg-interval)     ' generate averaging coefficient
       h := interval/sample #> 1     ' coefficient => 1 (immediate) but <= 1000 (real slow)
       h <#= 1000
       avg += (interval-avg)/h       ' accumulate new sample with response dependant upon change
     DisplayCAP(avg)


pub DisplayCAP(n) | code,i,j,rem
' display cap code as well
     n *= 4                     ' rough scaling to pF - seems very close
     code := n
     i~
     j := 1
     repeat until code < 90     ' round off threshold is less than 99 for two digits
                                ' and less again for a clean code
       code /= 10
       i++                      ' count trailing digits until only 2 significant are left
       j *= 10                  ' track round off scale

     rem := n // j
 '    rem += rem/10              ' add in a tenth for rounding up
     if rem > j/2                ' round up code
       code++
     code := (code*10)+i          ' add in multiplier digit
     screen.scale(2,2)
     screen.number(460,174,code,-1,4,$20)
' display large reading
     n := DisplayUnits(n)
     screen.pen(1)
     screen.scale(3,5)
     screen.number(400,240,n,dp,8,$20) ' (x,y,val,decimals,digits,leader)

pub DisplayUnits(val) | i
   screen.pen(1)
   screen.scale(1,2)
   if val > 800000
     result := val/10000        ' retain 2 decimal places
     dp := 2
     screen.text(458,256,string("uF"))
     repeat i from 128 to 192
       screen.color(i,$ffCC)
   elseif val > 8000
     result := val/10
     dp := 2
     screen.text(458,256,string("nF"))
     repeat i from 128 to 192
       screen.color(i,$ffC0)
   else
     screen.text(458,256,string("pF"))
     repeat i from 128 to 192
       screen.color(i,$ff40)
     result := val
     dp := -1


PUB ms(Period)                                    '1 mS
  if period
    waitcnt(_1mS * Period + cnt)


DAT
                        org
Measure
                        mov     ar0,par
                        add     ar0,#4
                        mov     dira,cap
                        mov     outa,#0
                        mov     r0,cnt
mloop
                        mov     dira,cap                ' discharge cap
                        mov     r0,cnt
                        add     r0,dtime
                        waitcnt r0,#0
                        mov     dira,#0                 ' charge cap through 10K only
                        mov     r0,cnt                  '
                        waitpeq cap,cap                 ' wait for charge
                        mov     r1,cnt                  ' capture the cnt on a lo to hi
                        sub     r1,r0                   ' calculate time diff
                        wrlong  r1,par                  ' update the interval
                        wrlong  update,ar0

                        jmp     #mloop

update                  long -1
dtime                   long 80_000*100
#ifdef demoboard
cap                     long |<cappin
#else
cap                     long |<ps2clk
#endif

r0                      res 1
r1                      res 1
r2                      res 1
ar0                     res 1


con
#ifdef demoboard
  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000
#else
  _clkmode = xtal1 + pll8x
  _xinfreq = 10_000_000
#endif
  clockfreq = ((_CLKMODE - XTAL1) >> 6) * _XINFREQ


  _1uS  = clockfreq / 1_000_000 'Divisor for  1 uS
  _10uS = clockfreq /   100_000 'Divisor for 10 uS
  _1mS  = clockfreq /     1_000 'Divisor for  1 mS
  _1S   = clockfreq /         1 'Divisor for  1 S

#ifdef demoboard
  cappin        = 0              ' adjust this on the demoboard
  video         = 16



#else
' Pin assignments for CE0972 Widget
' RS485
  tr1           = 0
  te1           = 1
  tr2           = 2
  te2           = 3

' RS232
  dsr232        = 4             '->IN resistive input
  ri232         = 5             '->IN resistive input
  dcd232        = 6             '->IN resistive input
  rxd232        = 7             '->IN receive input
  txd232        = 8             '<-OUT transmit output  --> rxd
  cts232        = 9             '->IN cts input
  rts232        = 10            '<-OUT rts output       --> cts
  dtr232        = 11            '<-OUT dtr output (through opamp)  -->dsr,dcd

' LAN
  lantxd        = 12
  lanrxd        = 13

'FIBER
  fibtxd        = 14
  fibrxd        = 15

  video         = 16
  audio         = 24
  spiCS         = 25
  ps2clk        = 26
  ps2dat        = 27
  scl           = 28
  sda           = 29
  txdusb        = 30
  rxdusb        = 31

  spiCLK        = scl
  spiDI         = sda
  spiDO         = ri232         ' data from SD

  cappin        = ps2clk
#endif

CON

  cr            = $0d
  lf            = $0a

obj
   screen       : "vgagraphicsfonts"


var

    long  dp,interval,status









